@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007-2017 VMware, Inc. All rights reserved.
+ * Copyright (c) 2007-2021,2023 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -72,9 +72,9 @@ static char implicitReadToken;
 
 typedef struct parse_table
 {
-   int    type;
-   char  *name;
-   void  *valuePtr;
+   int         type;
+   const char *name;
+   void       *valuePtr;
 } ParseTable;
 
 /*
@@ -753,12 +753,12 @@ FileLockScanDirectory(const char *lockDir,      // IN:
    /* Pass 2: Handle the 'M' entries */
    for (i = 0, err = 0; i < numEntries; i++) {
       LockValues *ptr;
-      Bool       myLockFile;
+      Bool myLockFile;
       LockValues memberValues;
-      char       buffer[FILELOCK_DATA_SIZE];
+      char buffer[FILELOCK_DATA_SIZE];  // Must be near memberValues,
+                                        // as it will be pointed by it
 
-      if ((fileList[i] == NULL) ||
-          (*fileList[i] == 'E')) {
+      if ((fileList[i] == NULL) || (*fileList[i] == 'E')) {
          continue;
       }
 
@@ -939,15 +939,21 @@ FileLockScanner(const char *lockDir,     // IN:
          }
 
          if (remove) {
-            if (prev == NULL) {
-               myValues->lockList = ptr->next;
-            } else {
-               prev->next = ptr->next;
-            }
-         }
+            ActiveLock *temp = ptr;
 
-         prev = ptr;
-         ptr = ptr->next;
+            ptr = ptr->next;
+            Posix_Free(temp->dirName);
+            Posix_Free(temp);
+
+            if (prev == NULL) {
+               myValues->lockList = ptr;
+            } else {
+               prev->next = ptr;
+            }
+         } else {
+            prev = ptr;
+            ptr = ptr->next;
+         }
       }
 
       FileSleeper(FILELOCK_PROGRESS_SAMPLE,
@@ -992,10 +998,9 @@ FileUnlockIntrinsic(FileLockToken *tokenPtr)  // IN:
 
    ASSERT(tokenPtr && (tokenPtr->signature == FILELOCK_TOKEN_SIGNATURE));
 
-   LOG(1, ("Requesting unlock on %s\n", tokenPtr->pathName));
+   LOG(1, "Requesting unlock on %s\n", tokenPtr->pathName);
 
    if (tokenPtr->portable) {
-
       /*
        * If the lockFilePath (a pointer) is the fixed-address token representing
        * an implicit read lock, there is no lock file and the token can simply
@@ -1041,9 +1046,8 @@ FileUnlockIntrinsic(FileLockToken *tokenPtr)  // IN:
           *    ENOENT: other locked + unlocked (w/ implicit unlink) file
           */
          if (Err_Errno() == EBUSY || Err_Errno() == ENOENT) {
-            LOG(0, ("Tolerating %s on unlink of advisory lock at %s\n",
-                    Err_Errno() == EBUSY ? "EBUSY" : "ENOENT",
-                    tokenPtr->pathName));
+            LOG(0, "Tolerating %s on unlink of advisory lock at %s\n",
+                Err_Errno() == EBUSY ? "EBUSY" : "ENOENT", tokenPtr->pathName);
          } else {
             err = Err_Errno();
             if (vmx86_debug) {
@@ -1099,13 +1103,9 @@ FileLockWaitForPossession(const char *lockDir,       // IN:
                            myValues->memberName) < 0))) &&
         ((strcmp(memberValues->lockType, LOCK_EXCLUSIVE) == 0) ||
          (strcmp(myValues->lockType, LOCK_EXCLUSIVE) == 0))) {
-      char *path;
-      Bool   thisMachine;
-
-      thisMachine = FileLockMachineIDMatch(myValues->machineID,
-                                           memberValues->machineID);
-
-      path = Unicode_Join(lockDir, DIRSEPS, fileName, NULL);
+      Bool thisMachine = FileLockMachineIDMatch(myValues->machineID,
+                                                memberValues->machineID);
+      char *path = Unicode_Join(lockDir, DIRSEPS, fileName, NULL);
 
       while ((err = FileLockSleeper(myValues)) == 0) {
          /* still there? */
@@ -1140,10 +1140,10 @@ FileLockWaitForPossession(const char *lockDir,       // IN:
           (err == EAGAIN)) {
          if (thisMachine) {
             Log(LGPFX" %s timeout on '%s' due to a local process '%s'\n",
-                    __FUNCTION__, path, memberValues->executionID);
+                __FUNCTION__, path, memberValues->executionID);
          } else {
             Log(LGPFX" %s timeout on '%s' due to another machine '%s'\n",
-                    __FUNCTION__, path, memberValues->machineID);
+                __FUNCTION__, path, memberValues->machineID);
          }
       }
 
@@ -1260,8 +1260,8 @@ FileLockCreateEntryDirectory(const char *lockDir,    // IN:
                              char **memberFilePath,  // OUT:
                              char **memberName)      // OUT:
 {
-   int err = 0;
-   uint32 randomNumber = 0;
+   int err;
+   VmTimeType startTimeMsec;
 
    ASSERT(lockDir != NULL);
 
@@ -1271,36 +1271,34 @@ FileLockCreateEntryDirectory(const char *lockDir,    // IN:
    *memberName = NULL;
 
    /* Fun at the races */
+   startTimeMsec = Hostinfo_SystemTimerMS();
 
    while (TRUE) {
       char *temp;
       FileData fileData;
+      VmTimeType ageMsec;
+      uint32 randomNumber;
 
       err = FileAttributesRobust(lockDir, &fileData);
       if (err == 0) {
         /* The name exists. Deal with it... */
 
-        if (fileData.fileType == FILE_TYPE_REGULAR) {
+        if (fileData.fileType != FILE_TYPE_DIRECTORY) {
            /*
-            * It's a file. Assume this is an (active?) old style lock and
-            * err on the safe side - don't remove it (and automatically
-            * upgrade to a new style lock).
+            * Locks are implemented via directories. Ancient hosted locks and
+            * lock files imported alongsize an ESXi VM are implemented as
+            * files. It's safe to remove these. The ESXi lock files are
+            * meaningless and the ancient hosted lock files have been dead
+            * for over a decade (at the time of writing this comment).
             */
 
-            Log(LGPFX" %s: '%s' exists; an old style lock file?\n",
+           err = FileDeletionRobust(lockDir, FALSE);
+           if (err == 0) {
+              Warning(LGPFX" %s: '%s' is not a directory. Removed.\n",
                       __FUNCTION__, lockDir);
-
-            err = EBUSY;
-            break;
-        }
-
-        if (fileData.fileType != FILE_TYPE_DIRECTORY) {
-           /* Not a directory; attempt to remove the debris */
-           if (FileDeletionRobust(lockDir, FALSE) != 0) {
-              Warning(LGPFX" %s: '%s' exists and is not a directory.\n",
-                      __FUNCTION__, lockDir);
-
-              err = ENOTDIR;
+           } else {
+              Warning(LGPFX" %s: an attempt to remove '%s' failed: %s\n",
+                      __FUNCTION__, lockDir, Err_Errno2String(err));
               break;
            }
 
@@ -1367,7 +1365,14 @@ FileLockCreateEntryDirectory(const char *lockDir,    // IN:
              }
          }
 
-         FileRemoveDirectoryRobust(*entryDirectory);
+         err = FileRemoveDirectoryRobust(*entryDirectory);
+
+         if (err != 0) {
+            Warning(LGPFX" %s unable to remove '%s': %s\n",
+                    __FUNCTION__, *entryDirectory, Err_Errno2String(err));
+
+            break;
+         }
       } else {
           if ((err != EEXIST) &&  // Another process/thread created it...
               (err != ENOENT)) {  // lockDir is gone...
@@ -1387,6 +1392,21 @@ FileLockCreateEntryDirectory(const char *lockDir,    // IN:
       *entryFilePath = NULL;
       *memberFilePath = NULL;
       *memberName = NULL;
+
+      /*
+       * If we've been trying to get the locking started for a unacceptable
+       * amount of time, bail. Something is seriously wrong, probably the
+       * file system or networking. Nothing we can do about it.
+       */
+
+      ageMsec = Hostinfo_SystemTimerMS() - startTimeMsec;
+
+      if (ageMsec > FILELOCK_PROGRESS_DEARTH) {
+         Warning(LGPFX" %s lack of progress on '%s'\n", __FUNCTION__, lockDir);
+
+         err = EBUSY;
+         break;
+      }
    }
 
    if (err != 0) {
@@ -1717,7 +1737,7 @@ FileLockIntrinsicPortable(const char *pathName,   // IN:
       goto bail;
    }
 
-   /* what is max(Number[1]... Number[all lockers])? */
+   /* What is max(Number[1]... Number[all lockers])? */
    *err = FileLockScanner(lockDir, FileLockNumberScan, myValues, FALSE);
 
    if (*err != 0) {
@@ -1738,7 +1758,9 @@ FileLockIntrinsicPortable(const char *pathName,   // IN:
                                    memberFilePath);
 
    /* Remove entry directory; it has done its job */
-   FileRemoveDirectoryRobust(entryDirectory);
+   if (*err == 0) {
+      *err = FileRemoveDirectoryRobust(entryDirectory);
+   }
 
    if (*err != 0) {
       /* clean up */
@@ -1750,8 +1772,7 @@ FileLockIntrinsicPortable(const char *pathName,   // IN:
    }
 
    /* Attempt to acquire the lock */
-   *err = FileLockScanner(lockDir, FileLockWaitForPossession,
-                          myValues, TRUE);
+   *err = FileLockScanner(lockDir, FileLockWaitForPossession, myValues, TRUE);
 
    switch (*err) {
    case 0:
@@ -1847,8 +1868,8 @@ FileLockIntrinsic(const char *pathName,    // IN:
    myValues.maxWaitTimeMsec = maxWaitTimeMsec;
 
    if (File_SupportsMandatoryLock(pathName)) {
-      LOG(1, ("Requesting %s lock on %s (mandatory, %u).\n",
-          myValues.lockType, pathName, myValues.maxWaitTimeMsec));
+      LOG(1, "Requesting %s lock on %s (mandatory, %u).\n",
+          myValues.lockType, pathName, myValues.maxWaitTimeMsec);
 
       tokenPtr = FileLockIntrinsicMandatory(pathName, lockBase, &myValues, err);
    } else {
@@ -1858,9 +1879,9 @@ FileLockIntrinsic(const char *pathName,    // IN:
       myValues.locationChecksum = FileLockLocationChecksum(lockBase); // free this!
       myValues.memberName = NULL;
 
-      LOG(1, ("Requesting %s lock on %s (%s, %s, %u).\n",
+      LOG(1, "Requesting %s lock on %s (%s, %s, %u).\n",
           myValues.lockType, pathName, myValues.machineID,
-          myValues.executionID, myValues.maxWaitTimeMsec));
+          myValues.executionID, myValues.maxWaitTimeMsec);
 
       tokenPtr = FileLockIntrinsicPortable(pathName, lockBase, &myValues, err);
 
@@ -1917,11 +1938,11 @@ FileLockIsLockedMandatory(const char *lockFile,  // IN:
    result = FileIOCreateRetry(&desc, lockFile, access, FILEIO_OPEN, 0644, 0);
 
    if (FileIO_IsSuccess(result)) {
-      Bool ret;
+      Bool success;
 
-      ret = !FileIO_IsSuccess(FileIO_Close(&desc));
+      success = FileIO_IsSuccess(FileIO_Close(&desc));
 
-      ASSERT(!ret);
+      ASSERT(success);
       return FALSE;
    } else if (result == FILEIO_LOCK_FAILED) {
       return TRUE;   // locked
