@@ -1,5 +1,6 @@
 /*********************************************************
- * Copyright (C) 2002-2021 VMware, Inc. All rights reserved.
+ * Copyright (c) 2002-2024 Broadcom. All Rights Reserved.
+ * The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -84,6 +85,14 @@
  */
 #define CONFNAME_APPINFO_DISABLED "disabled"
 
+/**
+ * Defines the configuration to remove duplicate applications.
+ *
+ * @param boolean Set to TRUE to remove duplicate apps.
+ *                Set to FALSE to keep duplicate apps.
+ */
+#define CONFNAME_APPINFO_REMOVE_DUPLICATES "remove-duplicates"
+
 /**
  * Defines the configuration to use the WMI for getting the application
  * version information.
@@ -101,6 +110,82 @@
  ******************************************************************************
  */
 
+/*
+ ******************************************************************************
+ * BEGIN containerInfo goodies.
+ */
+
+/**
+ * Defines the string used for the ContainerInfo config file group.
+ */
+#define CONFGROUPNAME_CONTAINERINFO "containerinfo"
+
+/**
+ * Define a custom ContainerInfo poll interval (in seconds).
+ *
+ * @note Illegal values result in a @c g_warning and fallback to the default
+ * poll interval.
+ *
+ * @param int   User-defined poll interval.  Set to 0 to disable polling.
+ */
+#define CONFNAME_CONTAINERINFO_POLLINTERVAL "poll-interval"
+
+/**
+ * Define the limit on the maximum number of containers to collect info from.
+ * If the number of running containers exceeds the limit, only the most recently
+ * created containers will be published.
+ *
+ * @note Illegal values result in a @c g_warning and fallback to the default
+ * max container limit.
+ *
+ * @param int   User-defined max limit for # of containers queried.
+ */
+#define CONFNAME_CONTAINERINFO_LIMIT "max-containers"
+
+/**
+ * Defines the configuration to remove duplicate containers.
+ *
+ * @param boolean Set to TRUE to remove duplicate containers.
+ *                Set to FALSE to keep duplicate containers.
+ */
+#define CONFNAME_CONTAINERINFO_REMOVE_DUPLICATES "remove-duplicates"
+
+/**
+ * Define the docker unix socket to use to communicate with the docker daemon.
+ *
+ * @note Illegal values result in a @c g_warning and fallback to docker's
+ * default unix socket.
+ *
+ * @param string  absolute file path of the docker unix socket in use.
+ */
+#define CONFNAME_CONTAINERINFO_DOCKERSOCKET "docker-unix-socket"
+
+/**
+ * Define the containerd unix socket to connect with containerd gRPC server.
+ * This should be used to get containers.
+ *
+ * @note Illegal values result in a @c g_warning and fallback to the containerd
+ * default unix socket.
+ *
+ * @param string  absolute file path of the containerd unix socket in use.
+ */
+#define CONFNAME_CONTAINERINFO_CONTAINERDSOCKET "containerd-unix-socket"
+
+/**
+ * Define the list of namespaces to be queried for the running containers.
+ *
+ * @note Illegal values result in a @c g_warning and fallback to the default
+ * list of namespaces.
+ *
+ * @param string  Comma separated list of namespaces to be queried.
+ */
+#define CONFNAME_CONTAINERINFO_ALLOWED_NAMESPACES "allowed-namespaces"
+
+/*
+ * END containerInfo goodies.
+ ******************************************************************************
+ */
+
 /*
  ******************************************************************************
  * BEGIN ServiceDiscovery goodies.
@@ -127,6 +212,34 @@
  ******************************************************************************
  */
 
+/*
+ ******************************************************************************
+ * BEGIN ComponentMgr goodies.
+ */
+
+/**
+ * Defines the current poll interval (in seconds).
+ * This value is controlled by the componentMgr.poll-interval config file
+ * option.
+ */
+#define COMPONENTMGR_CONF_POLLINTERVAL "poll-interval"
+
+/**
+ * Name of section of configuration to be read from tools.conf.
+ */
+#define COMPONENTMGR_CONF_GROUPNAME "componentmgr"
+
+/**
+ * Defines the components  managed by the componentMgr plugin.
+ * This value is controlled by the componentMgr.included config file option.
+ */
+#define COMPONENTMGR_CONF_INCLUDEDCOMPONENTS "included"
+
+/*
+ * END ComponentMgr goodies.
+ ******************************************************************************
+ */
+
 /*
  ******************************************************************************
  * BEGIN GuestStore upgrader goodies.
@@ -163,16 +276,6 @@
  */
 #define CONFNAME_GSUPGRADE_POLLINTERVAL "poll-interval"
 
-/**
- * Define a custom GuestStore periodic Upgrade interval (in seconds).
- *
- * @note Illegal values result in a @c g_warning and fallback to the default
- * upgrade interval.
- *
- * @param int   User-defined upgrade interval.  Set to 0 to disable polling.
- */
-#define CONFNAME_GSUPGRADE_UPGRADEINTERVAL "upgrade-interval"
-
 /**
  * Define a custom GuestStore content path prefix.
  *
@@ -465,7 +568,7 @@
 
 /**
  * Lets users configure the process timeout value in deployPkg
- * Valid value range: 0x01 ~ 0xFF
+ * Valid value range: 1 ~ 3600
  */
 #define CONFNAME_DEPLOYPKG_PROCESSTIMEOUT "process-timeout"
 
@@ -475,6 +578,12 @@
  */
 #define CONFNAME_DEPLOYPKG_ENABLE_CUST "enable-customization"
 
+/**
+ * How long does guest customization wait until cloud-init execution done
+ * Valid value range: 0 ~ 1800
+ */
+#define CONFNAME_DEPLOYPKG_WAIT_CLOUDINIT_TIMEOUT "wait-cloudinit-timeout"
+
 /*
  * END deployPkg goodies.
  ******************************************************************************
@@ -482,7 +591,24 @@
 
 
 /** Where to find Tools data in the Win32 registry. */
-#define CONF_VMWARE_TOOLS_REGKEY    "Software\\VMware, Inc.\\VMware Tools"
+#if defined(_WIN32)
+#   ifndef WSTR
+#      define WSTR_(X) L##X
+#      define WSTR(X) WSTR_(X)
+#   endif
+#   define CONF_VMWARE_TOOLS_REGKEY  "Software\\VMware, Inc.\\VMware Tools"
+#   define CONF_VMWARE_TOOLS_INSTPATH_KEY "InstallPath"
+
+#   define CONF_VMWARE_TOOLS_REGKEY_W WSTR(CONF_VMWARE_TOOLS_REGKEY)
+#   define CONF_VMWARE_TOOLS_INSTPATH_KEY_W WSTR(CONF_VMWARE_TOOLS_INSTPATH_KEY)
+#   ifdef UNITY_FOR_VIEW
+#      define CONF_VMWARE_RDE_REGKEY  "SOFTWARE\\VMware, Inc.\\VMware VDM\\RemoteExperienceAgent"
+#      define CONF_VMWARE_RDE_INSTPATH_KEY "InstallPath"
+
+#      define CONF_VMWARE_RDE_REGKEY_W WSTR(CONF_VMWARE_RDE_REGKEY)
+#      define CONF_VMWARE_RDE_INSTPATH_KEY_W WSTR(CONF_VMWARE_RDE_INSTPATH_KEY)
+#   endif
+#endif
 
 /* Wait 5 seconds between polls to see if the conf file has changed */
 #define CONF_POLL_TIME     5
@@ -497,6 +623,7 @@
 #define CONFNAME_AUTOUPGRADE_ALLOW_UPGRADE "allow-upgrade"
 #define CONFNAME_AUTOUPGRADE_ALLOW_ADD_FEATURE "allow-add-feature"
 #define CONFNAME_AUTOUPGRADE_ALLOW_REMOVE_FEATURE "allow-remove-feature"
+#define CONFNAME_AUTOUPGRADE_ALLOW_MSI_TRANSFORMS "allow-msi-transforms"
 
 /*
  * END upgrader goodies.
@@ -560,6 +687,34 @@
  ******************************************************************************
  */
 
+/*
+ ******************************************************************************
+ * BEGIN AMSI Fileless goodies.
+ */
+
+/**
+ * Defines the string used for vsep plugin AMSI Fileless config file group
+ */
+#define CONFGROUPNAME_AMSI "giamsi"
+
+/* Default state of AMSI config*/
+#define VSEP_DEFAULT_AMSI_STATE TRUE
+
+/**
+ * Defines user-defined maximum AMSI client connections.
+ */
+#define CONFNAME_AMSI_MAX_CLIENT_CONNECTIONS "amsi-max-client-connections"
+
+/**
+ * Defines user-defined maximum script size supported in fileless.
+ */
+#define CONFNAME_AMSI_MAX_SCRIPT_SIZE_IN_BYTES "amsi-max-script-size-in-bytes"
+
+/*
+ * END Fileless goodies.
+ ******************************************************************************
+ */
+
 /*
  ******************************************************************************
  * BEGIN CarbonBlack helper plugin goodies.
@@ -642,4 +797,24 @@
  ******************************************************************************
  */
 
+/*
+ ******************************************************************************
+ * BEGIN timeSync plugin goodies.
+ */
+
+/**
+ * Defines the string used for the timeSync config file group.
+ */
+#define CONFGROUPNAME_TIMESYNC "timeSync"
+
+/**
+ * Enable (or disable) the TimeInfo API.
+ */
+#define CONFNAME_TIMESYNC_TIMEINFO_ENABLED "timeInfo.enabled"
+
+/*
+ * END timeSync goodies.
+ ******************************************************************************
+ */
+
 #endif /* __CONF_H__ */
