@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2017 VMware, Inc. All rights reserved.
+ * Copyright (c) 1998-2020,2023 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -87,42 +87,26 @@ typedef char *File_MakeTempCreateNameFunc(uint32 num,
                                           void *data);
 
 #if defined(__APPLE__)
-typedef enum {
-   FILEMACOS_UNMOUNT_SUCCESS,
-   FILEMACOS_UNMOUNT_SUCCESS_ALREADY,
-   FILEMACOS_UNMOUNT_ERROR,
-} FileMacosUnmountStatus;
-
-FileMacosUnmountStatus FileMacos_UnmountDev(char const *bsdDev,
-                                            Bool wholeDev,
-                                            Bool eject,
-                                            Bool su);
-
-void FileMacos_MountDevAsyncNoResult(char const *bsdDev,
-                                     Bool su);
-
 Bool FileMacos_IsOnSparseDmg(int fd);
-Bool FileMacos_IsSliceDevice(char const *bsdDev);
 
-char *FileMacos_DiskDevToUserFriendlyName(char const *bsdDiskDev);
-char *FileMacos_DiskDevToVolumeName(char const *bsdDiskDev);
-
-char *FileMacos_DiskDeviceToUniqueID(char const *bsdPath);
-char *FileMacos_UniqueIDToDiskDevice(char const *identifier);
 Bool FileMacOS_MakeSecureLibraryCopies(const char   *inDir,
                                        const char  **dylibName,
                                        unsigned      numDylibs,
                                        char        **outDir);
+
 #elif defined VMX86_SERVER
 struct FS_PartitionListResult;
 
 int File_GetVMFSAttributes(const char *pathName,
                            struct FS_PartitionListResult **fsAttrs);
+
 int File_GetVMFSFSType(const char *pathName,
                        int fd,
                        uint16 *fsTypeNum);
+
 int File_GetVMFSVersion(const char *pathName,
                         uint32 *versionNum);
+
 int File_GetVMFSBlockSize(const char *pathName,
                           uint32 *blockSize);
 
@@ -132,6 +116,13 @@ int File_GetVMFSMountInfo(const char *pathName,
                           char **remoteIP,
                           char **remoteMountPoint,
                           char **localMountPoint);
+
+int File_GetVMFSLockInfo(const char *path,
+                         uint32 *outOpenFlags,
+                         uint32 *outWorldID,
+                         char **outWorldName,
+                         char **outVMFSMacAddr,
+                         uint32 *outVMFSLockMode);
 #endif
 
 Bool File_SupportsZeroedThick(const char *pathName);
@@ -172,24 +163,27 @@ char *File_PathJoin(const char *dirName,
 Bool File_CreateDirectory(const char *pathName);
 
 Bool File_CreateDirectoryEx(const char *pathName,
-                            int mask);
+                            int mode);
 
 Bool File_EnsureDirectory(const char *pathName);
 
+Bool File_EnsureDirectoryEx(const char *pathName,
+                            int mode);
+
 Bool File_DeleteEmptyDirectory(const char *pathName);
 
 Bool File_CreateDirectoryHierarchy(const char *pathName,
                                    char **topmostCreated);
 
 Bool File_CreateDirectoryHierarchyEx(const char *pathName,
-                                     int mask,
+                                     int mode,
                                      char **topmostCreated);
 
 Bool File_DeleteDirectoryContent(const char *pathName);
 
 Bool File_DeleteDirectoryTree(const char *pathName);
 
-int File_ListDirectory(const char *pathName,
+int File_ListDirectory(const char *dirName,
                        char ***ids);
 
 Bool File_IsOsfsVolumeEmpty(const char *pathName);
@@ -202,10 +196,10 @@ char * File_StripFwdSlashes(const char *pathName);
  * Simple file-system walk.
  */
 
-WalkDirContext File_WalkDirectoryStart(const char *parentPath);
+WalkDirContext File_WalkDirectoryStart(const char *dirName);
 
 Bool File_WalkDirectoryNext(WalkDirContext context,
-                            char **path);
+                            char **fileName);
 
 void File_WalkDirectoryEnd(WalkDirContext context);
 
@@ -236,6 +230,10 @@ uint64 File_GetFreeSpace(const char *pathName,
 
 uint64 File_GetCapacity(const char *pathName);
 
+#ifdef _WIN32
+char *File_GetNTGlobalFinalPath(const char *pathName);
+#endif
+
 int File_MakeTempEx(const char *dir,
                     const char *pathName,
                     char **presult);
@@ -247,6 +245,7 @@ int File_MakeTempEx2(const char *dir,
                      char **presult);
 
 char *File_MakeSafeTempDir(const char *prefix);
+char *File_MakeSafeTempSubdir(const char *safeDir, const char *subdirName);
 
 int64 File_GetModTime(const char *pathName);
 
@@ -254,7 +253,8 @@ char *File_GetModTimeString(const char *pathName);
 
 char *File_GetUniqueFileSystemID(const char *pathName);
 
-char *File_GetMountPath(const char *pathName, Bool checkEntirePath);
+char *File_GetMountPath(const char *pathName,
+                        Bool checkEntirePath);
 
 #ifdef _WIN32
 char *File_GetVolumeGUID(const char *pathName);
@@ -372,11 +372,19 @@ char *File_ExpandAndCheckDir(const char *dirName);
 
 char *File_GetSafeTmpDir(Bool useConf);
 
+char *File_GetSafeRandomTmpDir(Bool useConf);
+
+char *File_GetUacSafeRandomTmpDir(Bool useConf);
+
 int File_MakeSafeTemp(const char *tag,
                       char **presult);
 
 Bool File_DoesVolumeSupportAcls(const char *pathName);
 
+Bool File_IsSubPathOf(const char *base, const char *path);
+
+Bool File_DoesVolumeSupportConvertBlocks(const char *pathName);
+
 /*
  *---------------------------------------------------------------------------
  *
