@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2013-2021 VMware, Inc. All rights reserved.
+ * Copyright (C) 2013-2024 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -75,7 +75,7 @@ extern "C" {
  */
 
 #if defined __GNUC__
-#define _DMB(t) asm volatile("dmb " #t ::: "memory")
+#define _DMB(t) asm volatile ("dmb " #t ::: "memory")
 #elif defined _MSC_VER
 #define _DMB(t) __dmb(_ARM64_BARRIER_##t)
 #else
@@ -103,7 +103,7 @@ extern "C" {
  */
 
 #if defined __GNUC__
-#define _DSB(t) asm volatile("dsb " #t ::: "memory")
+#define _DSB(t) asm volatile ("dsb " #t ::: "memory")
 #elif defined _MSC_VER
 #define _DSB(t) __dsb(_ARM64_BARRIER_##t)
 #else
@@ -130,11 +130,11 @@ extern "C" {
  *----------------------------------------------------------------------
  */
 
-static INLINE void
+static inline void
 ISB(void)
 {
 #if defined __GNUC__
-   asm volatile("isb" ::: "memory");
+   asm volatile ("isb" ::: "memory");
 #elif defined _MSC_VER
    __isb(_ARM64_BARRIER_SY);
 #else
@@ -143,6 +143,48 @@ ISB(void)
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * ESB --
+ *
+ *      Error synchronization barrier.
+ *
+ *      Error synchronization event as per Arm ARM. NOP if ARMv8.2
+ *      RAS extensions are not implemented.
+ *
+ * Results:
+ *      None
+ *
+ * Side effects:
+ *      SError exception or DISR/VDISR getting updated.
+ *
+ *----------------------------------------------------------------------
+ */
+
+#if defined __GNUC__
+static inline void
+ESB(void)
+{
+   /*
+    * The assembler of gcc 9.3 with -march=armv8-a errors out with
+    * "Error: selected processor does not support `esb'"
+    * There is no way to cleanly temporarily push/pop -march=armv8.2-a or the
+    * ras extension. The error does not occur with gcc versions >= 10.2.
+    */
+# if __GNUC__ > 10 || __GNUC__ == 10 && __GNUC_MINOR__ >= 2
+   asm volatile("esb" ::: "memory");
+# else
+   asm volatile(
+      ".arch armv8.2-a"       "\n\t"
+      "esb"                   "\n\t"
+      ".arch " XSTR(VMW_ARCH)
+      ::: "memory"
+   );
+# endif
+}
+#endif
+
 /*
  * Memory Barriers
  * ===============
@@ -154,46 +196,66 @@ ISB(void)
  * accesses accross the barrier. It is not a CPU instruction, it is a compiler
  * directive (i.e. it does not emit any code).
  *
+ * => A compiler memory barrier on its own is useful for coordinating
+ *    with an interrupt handler (or preemption logic in the scheduler)
+ *    on the same CPU, so that the order of read and write
+ *    instructions in code that might be interrupted is consistent
+ *    with the barriers. But when there are other CPUs involved, or
+ *    other types of devices like memory-mapped I/O and DMA
+ *    controllers, a compiler memory barrier is not enough.
+ *
  * A CPU memory barrier prevents the CPU from re-ordering memory accesses
  * accross the barrier. It is a CPU instruction.
  *
+ * => On its own the CPU instruction isn't useful because the compiler
+ *    may reorder loads and stores around the CPU instruction.  It is
+ *    useful only when combined with a compiler memory barrier.
+ *
  * A memory barrier is the union of a compiler memory barrier and a CPU memory
- * barrier. A compiler memory barrier is a useless construct by itself. It is
- * only useful when combined with a CPU memory barrier, to implement a memory
  * barrier.
  *
  *    Semantics
  *    ---------
  *
- * At the time COMPILER_*_BARRIER were created (and references to them were
+ * At the time COMPILER_MEM_BARRIER was created (and references to it were
  * added to the code), the code was only targetting x86. The intent of the code
  * was really to use a memory barrier, but because x86 uses a strongly ordered
- * memory model, the CPU would not re-order memory accesses, and the code could
- * get away with using just a compiler memory barrier. So COMPILER_*_BARRIER
- * were born and were implemented as compiler memory barriers _on x86_. But
- * make no mistake, _the semantics that the code expects from
- * COMPILER_*_BARRIER are that of a memory barrier_!
+ * memory model, the CPU would not re-order most memory accesses (store-load
+ * ordering still requires MFENCE even on x86), and the code could get away
+ * with using just a compiler memory barrier. So COMPILER_MEM_BARRIER was born
+ * and was implemented as a compiler memory barrier _on x86_. But make no
+ * mistake, _the semantics that the code expects from COMPILER_MEM_BARRIER is
+ * that of a memory barrier_!
  *
  *    DO NOT USE!
  *    -----------
  *
- * On at least one non-x86 architecture, COMPILER_*_BARRIER are
- * 1) Misnomers
+ * On at least one non-x86 architecture, COMPILER_MEM_BARRIER is
+ * 1) A misnomer
  * 2) Not fine-grained enough to provide the best performance.
- * For the above two reasons, usage of COMPILER_*_BARRIER is now deprecated.
- * _Do not add new references to COMPILER_*_BARRIER._ Instead, precisely
+ * For the above two reasons, usage of COMPILER_MEM_BARRIER is now deprecated.
+ * _Do not add new references to COMPILER_MEM_BARRIER._ Instead, precisely
  * document the intent of your code by using
  * <mem_type/purpose>_<before_access_type>_BARRIER_<after_access_type>.
- * Existing references to COMPILER_*_BARRIER are being slowly but surely
- * converted, and when no references are left, COMPILER_*_BARRIER will be
+ * Existing references to COMPILER_MEM_BARRIER are being slowly but surely
+ * converted, and when no references are left, COMPILER_MEM_BARRIER will be
  * retired.
  *
  * Thanks for pasting this whole comment into every architecture header.
  */
 
-#define COMPILER_READ_BARRIER()  SMP_R_BARRIER_R()
-#define COMPILER_WRITE_BARRIER() SMP_W_BARRIER_W()
-#define COMPILER_MEM_BARRIER()   SMP_RW_BARRIER_RW()
+/*
+ * To match x86 TSO semantics, we need to guarantee ordering for
+ * everything _except_ store-load:
+ *
+ * - DMB ISHLD orders load-load and load-store.
+ * - DMB ISHST orders store-store.
+ *
+ * In contrast, SMP_RW_BARRIER_RW, or DMB ISH, orders all four
+ * (load-load, load-store, store-load, store-store), so it's stronger
+ * than we need -- like x86 MFENCE.
+ */
+#define COMPILER_MEM_BARRIER() do { _DMB(ISHLD); _DMB(ISHST); } while (0)
 
 /*
  * Memory barriers. These take the form of
@@ -201,7 +263,7 @@ ISB(void)
  * <mem_type/purpose>_<before_access_type>_BARRIER_<after_access_type>
  *
  * where:
- *   <mem_type/purpose> is either SMP, DMA, or MMIO.
+ *   <mem_type/purpose> is either INTR, SMP, DMA, or MMIO.
  *   <*_access type> is either R(load), W(store) or RW(any).
  *
  * Above every use of these memory barriers in the code, there _must_ be a
@@ -256,7 +318,7 @@ ISB(void)
 #define MMIO_RW_BARRIER_W()   MMIO_RW_BARRIER_RW()
 #define MMIO_RW_BARRIER_RW()  _DSB(SY)
 
-#ifndef _MSC_VER
+#ifdef __GNUC__
 
 /*
  * _GET_CURRENT_PC --
@@ -271,7 +333,7 @@ ISB(void)
  */
 
 #define _GET_CURRENT_PC(pc)                                                   \
-   asm volatile("1: adr %0, 1b" : "=r" (pc))
+   asm volatile ("1: adr %0, 1b" : "=r" (pc))
 
 static INLINE_ALWAYS void *
 GET_CURRENT_PC(void)
@@ -285,16 +347,15 @@ GET_CURRENT_PC(void)
 /*
  * GET_CURRENT_LOCATION --
  *
- * Updates the arguments with the values of the pc, x29, sp and x30
- * registers at the current code location where the macro is invoked.
+ * Updates the arguments with the values of the pc, fp, sp and the
+ * return address at the current code location where the macro is invoked.
  */
 
-#define GET_CURRENT_LOCATION(pc, fp, sp, lr) do {                             \
+#define GET_CURRENT_LOCATION(pc, fp, sp, retAddr) do {                        \
    _GET_CURRENT_PC(pc);                                                       \
-   asm volatile("mov %0, x29" "\n\t"                                          \
-                "mov %1, sp"  "\n\t"                                          \
-                "mov %2, x30"                                                 \
-                : "=r" (fp), "=r" (sp), "=r" (lr));                           \
+   asm volatile ("mov %0, sp" : "=r" (sp));                                   \
+   fp = (uint64)GetFrameAddr();                                               \
+   retAddr = (uint64)GetReturnAddress();                                      \
 } while (0)
 
 
@@ -316,7 +377,7 @@ GET_CURRENT_PC(void)
 
 #define MRS(name) ({                                                          \
    uint64 val;                                                                \
-   asm volatile("mrs %0, " XSTR(name) : "=r" (val) :: "memory");              \
+   asm volatile ("mrs %0, " XSTR(name) : "=r" (val) :: "memory");             \
    val;                                                                       \
 })
 
@@ -339,10 +400,12 @@ GET_CURRENT_PC(void)
  */
 
 #define MSR(name, val)                                                        \
-   asm volatile("msr " XSTR(name) ", %0" :: "r" (val) : "memory")
+   asm volatile ("msr " XSTR(name) ", %0" :: "r" (val) : "memory")
 
 #define MSR_IMMED(name, val)                                                  \
-   asm volatile("msr " XSTR(name) ", %0" :: "i" (val) : "memory")
+   asm volatile ("msr " XSTR(name) ", %0" :: "i" (val) : "memory")
+
+#endif // ifdef __GNUC__
 
 
 /*
@@ -358,12 +421,20 @@ GET_CURRENT_PC(void)
  *----------------------------------------------------------------------
  */
 
-static INLINE uint32
-MMIORead32(const volatile void *addr)
+static inline uint32
+MMIORead32(const volatile void *addr) // IN
 {
    uint32 res;
 
-   asm volatile ("ldr %w0, [%1]" : "=r" (res) : "r" (addr));
+#if defined __GNUC__
+   asm volatile ("ldr %w0, %1"
+                 : "=r" (res)
+                 : "m" (*(const volatile uint32 *)addr));
+#elif defined _MSC_VER
+   res = __iso_volatile_load32((const volatile __int32 *)addr);
+#else
+#error No compiler defined for MMIORead32
+#endif
    return res;
 }
 
@@ -381,12 +452,20 @@ MMIORead32(const volatile void *addr)
  *----------------------------------------------------------------------
  */
 
-static INLINE uint64
-MMIORead64(const volatile void *addr)
+static inline uint64
+MMIORead64(const volatile void *addr) // IN
 {
    uint64 res;
 
-   asm volatile ("ldr %x0, [%1]" : "=r" (res) : "r" (addr));
+#if defined __GNUC__
+   asm volatile ("ldr %x0, %1"
+                 : "=r" (res)
+                 : "m" (*(const volatile uint64 *)addr));
+#elif defined _MSC_VER
+   res = __iso_volatile_load64((const volatile __int64 *)addr);
+#else
+#error No compiler defined for MMIORead64
+#endif
    return res;
 }
 
@@ -401,11 +480,19 @@ MMIORead64(const volatile void *addr)
  *----------------------------------------------------------------------
  */
 
-static INLINE void
+static inline void
 MMIOWrite32(volatile void *addr, // OUT
-            uint32 val)
+            uint32 val)          // IN
 {
-   asm volatile ("str %w0, [%1]" : : "r" (val), "r" (addr) : "memory");
+#if defined __GNUC__
+   asm volatile ("str %w1, %0"
+                 : "=m" (*(volatile uint32 *)addr)
+                 : "r" (val));
+#elif defined _MSC_VER
+   __iso_volatile_store32((volatile __int32 *)addr, val);
+#else
+#error No compiler defined for MMIOWrite32
+#endif
 }
 
 
@@ -419,11 +506,19 @@ MMIOWrite32(volatile void *addr, // OUT
  *----------------------------------------------------------------------
  */
 
-static INLINE void
+static inline void
 MMIOWrite64(volatile void *addr, // OUT
-            uint64 val)
+            uint64 val)          // IN
 {
-   asm volatile ("str %x0, [%1]" : : "r" (val), "r" (addr) : "memory");
+#if defined __GNUC__
+   asm volatile ("str %x1, %0"
+                 : "=m" (*(volatile uint64 *)addr)
+                 : "r" (val));
+#elif defined _MSC_VER
+   __iso_volatile_store64((volatile __int64 *)addr, val);
+#else
+#error No compiler defined for MMIOWrite64
+#endif
 }
 
 
@@ -440,12 +535,20 @@ MMIOWrite64(volatile void *addr, // OUT
  *----------------------------------------------------------------------
  */
 
-static INLINE uint16
-MMIORead16(const volatile void *addr)
+static inline uint16
+MMIORead16(const volatile void *addr) // IN
 {
    uint16 res;
 
-   asm volatile ("ldrh %w0, [%1]" : "=r" (res) : "r" (addr));
+#if defined __GNUC__
+   asm volatile ("ldrh %w0, %1"
+                 : "=r" (res)
+                 : "m" (*(const volatile uint16 *)addr));
+#elif defined _MSC_VER
+   res = __iso_volatile_load16((const volatile __int16 *)addr);
+#else
+#error No compiler defined for MMIORead16
+#endif
    return res;
 }
 
@@ -460,11 +563,19 @@ MMIORead16(const volatile void *addr)
  *----------------------------------------------------------------------
  */
 
-static INLINE void
-MMIOWrite16(volatile void *addr,  // IN
+static inline void
+MMIOWrite16(volatile void *addr,  // OUT
             uint16 val)           // IN
 {
-   asm volatile ("strh %w0, [%1]" : : "r" (val), "r" (addr) : "memory");
+#if defined __GNUC__
+   asm volatile ("strh %w1, %0"
+                 : "=m" (*(volatile uint16 *)addr)
+                 : "r" (val));
+#elif defined _MSC_VER
+   __iso_volatile_store16((volatile __int16 *)addr, val);
+#else
+#error No compiler defined for MMIOWrite16
+#endif
 }
 
 
@@ -481,12 +592,20 @@ MMIOWrite16(volatile void *addr,  // IN
  *----------------------------------------------------------------------
  */
 
-static INLINE uint8
-MMIORead8(const volatile void *addr)
+static inline uint8
+MMIORead8(const volatile void *addr) // IN
 {
    uint8 res;
 
-   asm volatile ("ldrb %w0, [%1]" : "=r" (res) : "r" (addr));
+#if defined __GNUC__
+   asm volatile ("ldrb %w0, %1"
+                 : "=r" (res)
+                 : "m" (*(const volatile uint8 *)addr));
+#elif defined _MSC_VER
+   res = __iso_volatile_load8((const volatile __int8 *)addr);
+#else
+#error No compiler defined for MMIORead8
+#endif
    return res;
 }
 
@@ -501,14 +620,79 @@ MMIORead8(const volatile void *addr)
  *----------------------------------------------------------------------
  */
 
-static INLINE void
-MMIOWrite8(volatile void *addr, // IN
+static inline void
+MMIOWrite8(volatile void *addr, // OUT
            uint8 val)           // IN
 {
-   asm volatile ("strb %w0, [%1]" : : "r" (val), "r" (addr) : "memory");
+#if defined __GNUC__
+   asm volatile ("strb %w1, %0"
+                 : "=m" (*(volatile uint8 *)addr)
+                 : "r" (val));
+#elif defined _MSC_VER
+   __iso_volatile_store8((volatile __int8 *)addr, val);
+#else
+#error No compiler defined for MMIOWrite8
+#endif
 }
 
 
+#ifdef VM_HAS_INT128
+/*
+ *----------------------------------------------------------------------
+ *
+ * MMIORead128 --
+ *
+ *      IO read from address "addr".
+ *
+ * Results:
+ *      128-bit value at given location.
+ *
+ *----------------------------------------------------------------------
+ */
+static inline uint128
+MMIORead128(const volatile void *addr) // IN
+{
+   union {
+      uint128 val128;
+      struct {
+         uint64 val64[2];
+      };
+   } res;
+   asm volatile ("ldp %x0, %x1, %2"
+                 : "=r" (res.val64[0]), "=r" (res.val64[1])
+                 : "Q" (*(const volatile uint128 *)addr));
+   return res.val128;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * MMIOWrite128 --
+ *
+ *      IO write to address "addr".
+ *
+ *----------------------------------------------------------------------
+ */
+static inline void
+MMIOWrite128(volatile void *addr, // OUT
+             uint128 val)         // IN
+{
+   union {
+      uint128 val128;
+      struct {
+         uint64 val64[2];
+      };
+   } res = { .val128 = val };
+   asm volatile ("stp %x1, %x2, %0"
+                 : "=Q" (*(volatile uint128 *)addr)
+                 : "r" (res.val64[0]), "r" (res.val64[1]));
+}
+#endif // VM_HAS_INT128
+
+
+#ifdef __GNUC__
+
 /*
  *----------------------------------------------------------------------
  *
@@ -525,10 +709,10 @@ MMIOWrite8(volatile void *addr, // IN
  *----------------------------------------------------------------------
  */
 
-static INLINE void
+static inline void
 WFI(void)
 {
-   asm volatile("wfi" ::: "memory");
+   asm volatile ("wfi" ::: "memory");
 }
 
 
@@ -548,10 +732,10 @@ WFI(void)
  *----------------------------------------------------------------------
  */
 
-static INLINE void
+static inline void
 WFE(void)
 {
-   asm volatile("wfe" ::: "memory");
+   asm volatile ("wfe" ::: "memory");
 }
 
 
@@ -571,10 +755,10 @@ WFE(void)
  *----------------------------------------------------------------------
  */
 
-static INLINE void
+static inline void
 SEV(void)
 {
-   asm volatile("sev" ::: "memory");
+   asm volatile ("sev" ::: "memory");
 }
 
 
@@ -594,10 +778,10 @@ SEV(void)
  *-----------------------------------------------------------------------------
  */
 
-static INLINE void
+static inline void
 SetSPELx(VA va)
 {
-   asm volatile(
+   asm volatile (
       "msr     spsel, #1 \n\t"
       "mov     sp, %0    \n\t"
       "msr     spsel, #0 \n\t"
@@ -608,131 +792,6 @@ SetSPELx(VA va)
 }
 
 
-#ifndef MUL64_NO_ASM
-/*
- *-----------------------------------------------------------------------------
- *
- * Mul64x6464 --
- *
- *    Unsigned integer by fixed point multiplication, with rounding:
- *       result = floor(multiplicand * multiplier * 2**(-shift) + 0.5)
- *
- *       Unsigned 64-bit integer multiplicand.
- *       Unsigned 64-bit fixed point multiplier, represented as
- *         (multiplier, shift), where shift < 64.
- *
- * Result:
- *       Unsigned 64-bit integer product.
- *
- *-----------------------------------------------------------------------------
- */
-
-static INLINE uint64
-Mul64x6464(uint64 multiplicand,
-           uint64 multiplier,
-           uint32 shift)
-{
-   if (shift == 0) {
-      return multiplicand * multiplier;
-   } else {
-      uint64 lo, hi;
-
-      asm("mul   %0, %2, %3" "\n\t"
-          "umulh %1, %2, %3"
-          : "=&r" (lo), "=r" (hi)
-          : "r" (multiplicand), "r" (multiplier));
-      return (hi << (64 - shift) | lo >> shift) + (lo >> (shift - 1) & 1);
-   }
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * Muls64x64s64 --
- *
- *    Signed integer by fixed point multiplication, with rounding:
- *       result = floor(multiplicand * multiplier * 2**(-shift) + 0.5)
- *
- *       Signed 64-bit integer multiplicand.
- *       Unsigned 64-bit fixed point multiplier, represented as
- *         (multiplier, shift), where shift < 64.
- *
- * Result:
- *       Signed 64-bit integer product.
- *
- *-----------------------------------------------------------------------------
- */
-
-static INLINE int64
-Muls64x64s64(int64 multiplicand,
-             int64 multiplier,
-             uint32 shift)
-{
-   if (shift == 0) {
-      return multiplicand * multiplier;
-   } else {
-      uint64 lo, hi;
-
-      asm("mul   %0, %2, %3" "\n\t"
-          "smulh %1, %2, %3"
-          : "=&r" (lo), "=r" (hi)
-          : "r" (multiplicand), "r" (multiplier));
-      return (hi << (64 - shift) | lo >> shift) + (lo >> (shift - 1) & 1);
-   }
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * Mul64x3264 --
- *
- *    Unsigned integer by fixed point multiplication, with rounding:
- *       result = floor(multiplicand * multiplier * 2**(-shift) + 0.5)
- *
- *       Unsigned 64-bit integer multiplicand.
- *       Unsigned 32-bit fixed point multiplier, represented as
- *         (multiplier, shift), where shift < 64.
- *
- * Result:
- *       Unsigned 64-bit integer product.
- *
- *-----------------------------------------------------------------------------
- */
-
-static INLINE uint64
-Mul64x3264(uint64 multiplicand, uint32 multiplier, uint32 shift)
-{
-   return Mul64x6464(multiplicand, multiplier, shift);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * Muls64x32s64 --
- *
- *    Signed integer by fixed point multiplication, with rounding:
- *       result = floor(multiplicand * multiplier * 2**(-shift) + 0.5)
- *
- *       Signed 64-bit integer multiplicand.
- *       Unsigned 32-bit fixed point multiplier, represented as
- *         (multiplier, shift), where shift < 64.
- *
- * Result:
- *       Signed 64-bit integer product.
- *
- *-----------------------------------------------------------------------------
- */
-
-static INLINE int64
-Muls64x32s64(int64 multiplicand, uint32 multiplier, uint32 shift)
-{
-   return Muls64x64s64(multiplicand, multiplier, shift);
-}
-#endif
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -754,7 +813,7 @@ Muls64x32s64(int64 multiplicand, uint32 multiplier, uint32 shift)
  *-----------------------------------------------------------------------------
  */
 
-static INLINE void
+static inline void
 Div643232(uint64 dividend,   // IN
           uint32 divisor,    // IN
           uint32 *quotient,  // OUT
@@ -783,7 +842,7 @@ Div643232(uint64 dividend,   // IN
  *-----------------------------------------------------------------------------
  */
 
-static INLINE void
+static inline void
 Div643264(uint64 dividend,   // IN
           uint32 divisor,    // IN
           uint64 *quotient,  // OUT
@@ -810,7 +869,7 @@ Div643264(uint64 dividend,   // IN
  *----------------------------------------------------------------------
  */
 
-static INLINE void *
+static inline void *
 uint64set(void *dst, uint64 val, uint64 count)
 {
    void   *tmpDst = dst;
@@ -870,7 +929,7 @@ uint64set(void *dst, uint64 val, uint64 count)
  *-----------------------------------------------------------------------------
  */
 
-static INLINE void
+static inline void
 RDTSC_BARRIER(void)
 {
    ISB();
@@ -896,7 +955,7 @@ RDTSC_BARRIER(void)
  *-----------------------------------------------------------------------------
  */
 
-static INLINE void
+static inline void
 DCacheCleanInvalidate(VA va, uint64 len)
 {
    VA dva;
@@ -905,7 +964,7 @@ DCacheCleanInvalidate(VA va, uint64 len)
    for (dva = ROUNDDOWN(va, CACHELINE_SIZE);
         dva < va + len;
         dva += CACHELINE_SIZE) {
-      asm volatile("dc civac, %0" :: "r" (dva) : "memory");
+      asm volatile ("dc civac, %0" :: "r" (dva) : "memory");
    }
 
    /* Ensure completion. */
@@ -931,7 +990,7 @@ DCacheCleanInvalidate(VA va, uint64 len)
  *-----------------------------------------------------------------------------
  */
 
-static INLINE void
+static inline void
 DCacheClean(VA va, uint64 len)
 {
    VA dva;
@@ -940,14 +999,147 @@ DCacheClean(VA va, uint64 len)
    for (dva = ROUNDDOWN(va, CACHELINE_SIZE);
         dva < va + len;
         dva += CACHELINE_SIZE) {
-      asm volatile("dc cvac, %0" :: "r" (dva) : "memory");
+      asm volatile ("dc cvac, %0" :: "r" (dva) : "memory");
    }
 
    /* Ensure completion of clean. */
    _DSB(SY);
 }
 
-#endif // ifndef _MSC_VER
+#endif // ifdef __GNUC__
+
+#if defined _MSC_VER
+/* Until we implement Mul64x6464() with Windows intrinsics... */
+#define MUL64_NO_ASM 1
+#endif
+
+#ifdef MUL64_NO_ASM
+#include "mul64.h"
+#else
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Mul64x6464 --
+ *
+ *    Unsigned integer by fixed point multiplication, with rounding:
+ *       result = floor(multiplicand * multiplier * 2**(-shift) + 0.5)
+ *
+ *       Unsigned 64-bit integer multiplicand.
+ *       Unsigned 64-bit fixed point multiplier, represented as
+ *         (multiplier, shift), where shift < 64.
+ *
+ * Result:
+ *       Unsigned 64-bit integer product.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static inline uint64
+Mul64x6464(uint64 multiplicand,
+           uint64 multiplier,
+           uint32 shift)
+{
+   if (shift == 0) {
+      return multiplicand * multiplier;
+   } else {
+      uint64 lo, hi;
+
+      asm("mul   %0, %2, %3" "\n\t"
+          "umulh %1, %2, %3"
+          : "=&r" (lo), "=r" (hi)
+          : "r" (multiplicand), "r" (multiplier));
+      return (hi << (64 - shift) | lo >> shift) + (lo >> (shift - 1) & 1);
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Muls64x64s64 --
+ *
+ *    Signed integer by fixed point multiplication, with rounding:
+ *       result = floor(multiplicand * multiplier * 2**(-shift) + 0.5)
+ *
+ *       Signed 64-bit integer multiplicand.
+ *       Unsigned 64-bit fixed point multiplier, represented as
+ *         (multiplier, shift), where shift < 64.
+ *
+ * Result:
+ *       Signed 64-bit integer product.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static inline int64
+Muls64x64s64(int64 multiplicand,
+             int64 multiplier,
+             uint32 shift)
+{
+   if (shift == 0) {
+      return multiplicand * multiplier;
+   } else {
+      uint64 lo, hi;
+
+      asm("mul   %0, %2, %3" "\n\t"
+          "smulh %1, %2, %3"
+          : "=&r" (lo), "=r" (hi)
+          : "r" (multiplicand), "r" (multiplier));
+      return (hi << (64 - shift) | lo >> shift) + (lo >> (shift - 1) & 1);
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Mul64x3264 --
+ *
+ *    Unsigned integer by fixed point multiplication, with rounding:
+ *       result = floor(multiplicand * multiplier * 2**(-shift) + 0.5)
+ *
+ *       Unsigned 64-bit integer multiplicand.
+ *       Unsigned 32-bit fixed point multiplier, represented as
+ *         (multiplier, shift), where shift < 64.
+ *
+ * Result:
+ *       Unsigned 64-bit integer product.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static inline uint64
+Mul64x3264(uint64 multiplicand, uint32 multiplier, uint32 shift)
+{
+   return Mul64x6464(multiplicand, multiplier, shift);
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Muls64x32s64 --
+ *
+ *    Signed integer by fixed point multiplication, with rounding:
+ *       result = floor(multiplicand * multiplier * 2**(-shift) + 0.5)
+ *
+ *       Signed 64-bit integer multiplicand.
+ *       Unsigned 32-bit fixed point multiplier, represented as
+ *         (multiplier, shift), where shift < 64.
+ *
+ * Result:
+ *       Signed 64-bit integer product.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static inline int64
+Muls64x32s64(int64 multiplicand, uint32 multiplier, uint32 shift)
+{
+   return Muls64x64s64(multiplicand, multiplier, shift);
+}
+#endif
+
 
 #if defined __cplusplus
 } // extern "C"
