@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008-2016 VMware, Inc. All rights reserved.
+ * Copyright (c) 2008-2020,2023 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -29,7 +29,19 @@
  * @{
  */
 
+/*
+ * glib.h should not be placed inside `extern "C"' blocks.
+ * However, this header is often placed inside such blocks.
+ * Here we change back into C++ for glib.h
+ */
+#ifdef __cplusplus
+extern "C++" {
+#endif
 #include <glib.h>
+#ifdef __cplusplus
+}
+#endif
+
 #if defined(G_PLATFORM_WIN32)
 #  include <windows.h>
 #  include <objbase.h>
@@ -155,6 +167,15 @@ ToolsCore_LogState(guint level,
  */
 #define TOOLS_CORE_SIG_RESET  "tcs_reset"
 
+/**
+ * Signal sent when RpcChannel is going to be destroyed.
+ *
+ * @param[in]  src      The source object.
+ * @param[in]  ctx      ToolsAppCtx *: The application context.
+ * @param[in]  data     Client data.
+ */
+#define TOOLS_CORE_SIG_NO_RPC  "tcs_no_rpc"
+
 /**
  * Signal sent when a "set option" RPC message arrives.
  *
@@ -219,6 +240,26 @@ ToolsCore_LogState(guint level,
  */
 #define TOOLS_CORE_PROP_CTX "tcs_app_ctx"
 
+/**
+ * Event signaled when VMTools discovers a newer version is available.
+ *
+ * Name of the event that can be set to the notification event to
+ * indicate a new version of tools is available for install or upgrade.
+ */
+#define TOOLS_CORE_EVENTS_TOOLS_NEW_VERSION "VMToolsNewVersion"
+
+/**
+ * Event signaled when VMTools requires a system restart to complete an install.
+ *
+ * Name of the event that can be set to the notification event to
+ * indicate a system restart is required to complete the install or
+ * upgrade of tools.
+ */
+#define TOOLS_CORE_EVENTS_TOOLS_NEED_REBOOT "VMToolsNeedReboot"
+
+#define TOOLS_CORE_EVENTS_GLOBAL_SCOPE      "Global"
+
+
 
 /**
  * This enum lists all API versions that different versions of vmtoolsd support.
@@ -234,6 +275,18 @@ typedef enum {
 } ToolsCoreAPI;
 
 
+struct ToolsServiceProperty;
+
+/**
+ * Type of the function that installs a new property in
+ * the application context service object.
+ *
+ * @param[in] obj    The application context service object.
+ * @param[in] prop   Property to install.
+ */
+typedef void (*RegisterServiceProperty)(gpointer obj,
+                                        struct ToolsServiceProperty *prop);
+
 /**
  * Defines the context of a tools application. This data is provided by the
  * core services to applications when they're loaded.
@@ -259,6 +312,8 @@ typedef struct ToolsAppCtx {
 #else
    /** The FD to access the VMware blocking fs. -1 if no FD available. */
    int               blockFD;
+   /** The FD to access the uinput. -1 if no FD available. */
+   int               uinputFD;
    /** The native environment (without any VMware modifications). */
    const char      **envp;
 #endif
@@ -269,6 +324,13 @@ typedef struct ToolsAppCtx {
     * register and emit their own signals using this object.
     */
    gpointer          serviceObj;
+
+   /**
+    * Function pointer for plugins to register properties to
+    * the service object serviceObj.
+    * This allows a plugin to share data and services to others.
+    */
+   RegisterServiceProperty registerServiceProperty;
 } ToolsAppCtx;
 
 #if defined(G_PLATFORM_WIN32)
@@ -279,7 +341,7 @@ typedef struct ToolsAppCtx {
  *
  * @return TRUE if COM is initialized when the function returns.
  */
-G_INLINE_FUNC gboolean
+static inline gboolean
 ToolsCore_InitializeCOM(ToolsAppCtx *ctx)
 {
    if (!ctx->comInitialized) {
