@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2000,2014 VMware, Inc. All rights reserved.
+ * Copyright (C) 2000,2014,2018-2021,2023 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -37,9 +37,11 @@
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/kernel.h>
+#include <sys/lock.h>
 #include <sys/malloc.h>
 #include <sys/module.h>
 #include <sys/conf.h>
+#include <sys/rwlock.h>
 #include <sys/sysctl.h>
 
 #include <vm/vm.h>
@@ -62,7 +64,11 @@
 
 typedef struct {
    /* system structures */
+#if __FreeBSD_version >= 1300067
+   struct callout callout_handle;
+#else
    struct callout_handle callout_handle;
+#endif
 
    /* termination flag */
    volatile int stop;
@@ -82,6 +88,23 @@ typedef struct {
 
 MALLOC_DEFINE(M_VMMEMCTL, BALLOON_NAME, "vmmemctl metadata");
 
+/*
+ * FreeBSD specific MACROS
+ */
+#define VM_PAGE_LOCK(page) vm_page_lock(page);
+#define VM_PAGE_UNLOCK(page) vm_page_unlock(page)
+
+#define VM_OBJ_LOCK(object) VM_OBJECT_WLOCK(object)
+#define VM_OBJ_UNLOCK(object) VM_OBJECT_WUNLOCK(object);
+
+#define VM_SYS_PAGES vm_cnt.v_page_count
+
+#define KVA_ALLOC(size) kva_alloc(size);
+#define KVA_FREE(offset, size) kva_free(offset, size)
+
+#define KMEM_ALLOC(size) kmem_malloc(size, M_WAITOK | M_ZERO)
+#define KMEM_FREE(offset, size) kmem_free(offset, size)
+
 /*
  * Globals
  */
@@ -195,6 +218,10 @@ static __inline__ unsigned long os_ffz(unsigned long word)
    __asm__("bsfq %1,%0"
            :"=r" (word)
            :"r" (~word));
+#elif defined(__aarch64__)
+   __asm__("rbit %0, %1; clz %0,%0"
+           :"=r" (word)
+           :"r" (~word));
 #else
    __asm__("bsfl %1,%0"
            :"=r" (word)
@@ -223,7 +250,7 @@ static __inline__ unsigned long os_ffz(unsigned long word)
 unsigned long
 OS_ReservedPageGetLimit(void)
 {
-   return cnt.v_page_count;
+   return VM_SYS_PAGES;
 }
 
 
@@ -295,11 +322,7 @@ OS_ReservedPageGetHandle(PA64 pa)     // IN
 Mapping
 OS_MapPageHandle(PageHandle handle)     // IN
 {
-#if __FreeBSD_version < 1000000
-   vm_offset_t res = kmem_alloc_nofault(kernel_map, PAGE_SIZE);
-#else
-   vm_offset_t res = kva_alloc(PAGE_SIZE);
-#endif
+   vm_offset_t res = KVA_ALLOC(PAGE_SIZE);
 
    vm_page_t page = (vm_page_t)handle;
 
@@ -357,11 +380,7 @@ void
 OS_UnmapPage(Mapping mapping)           // IN
 {
    pmap_qremove((vm_offset_t)mapping, 1);
-#if __FreeBSD_version < 1000000
-   kmem_free(kernel_map, (vm_offset_t)mapping, PAGE_SIZE);
-#else
-   kva_free((vm_offset_t)mapping, PAGE_SIZE);
-#endif
+   KVA_FREE((vm_offset_t)mapping, PAGE_SIZE);
 }
 
 
@@ -369,7 +388,7 @@ static void
 os_pmap_alloc(os_pmap *p) // IN
 {
    /* number of pages (div. 8) */
-   p->size = (cnt.v_page_count + 7) / 8;
+   p->size = (VM_SYS_PAGES + 7) / 8;
 
    /*
     * expand to nearest word boundary
@@ -378,22 +397,14 @@ os_pmap_alloc(os_pmap *p) // IN
    p->size = (p->size + sizeof(unsigned long) - 1) &
                          ~(sizeof(unsigned long) - 1);
 
-#if __FreeBSD_version < 1000000
-   p->bitmap = (unsigned long *)kmem_alloc(kernel_map, p->size);
-#else
-   p->bitmap = (unsigned long *)kmem_malloc(kernel_arena, p->size, M_WAITOK | M_ZERO);
-#endif
+   p->bitmap = (unsigned long *)KMEM_ALLOC(p->size);
 }
 
 
 static void
 os_pmap_free(os_pmap *p) // IN
 {
-#if __FreeBSD_version < 1000000
-   kmem_free(kernel_map, (vm_offset_t)p->bitmap, p->size);
-#else
-   kmem_free(kernel_arena, (vm_offset_t)p->bitmap, p->size);
-#endif
+   KMEM_FREE((vm_offset_t)p->bitmap, p->size);
    p->size = 0;
    p->bitmap = NULL;
 }
@@ -466,12 +477,14 @@ os_kmem_free(vm_page_t page) // IN
    os_state *state = &global_state;
    os_pmap *pmap = &state->pmap;
 
-   if ( !vm_page_lookup(state->vmobject, page->pindex) ) {
-      return;
+   VM_OBJ_LOCK(state->vmobject);
+   if (vm_page_lookup(state->vmobject, page->pindex)) {
+      os_pmap_putindex(pmap, page->pindex);
+      VM_PAGE_LOCK(page);
+      vm_page_free(page);
+      VM_PAGE_UNLOCK(page);
    }
-
-   os_pmap_putindex(pmap, page->pindex);
-   vm_page_free(page);
+   VM_OBJ_UNLOCK(state->vmobject);
 }
 
 
@@ -483,8 +496,11 @@ os_kmem_alloc(int alloc_normal_failed) // IN
    os_state *state = &global_state;
    os_pmap *pmap = &state->pmap;
 
+   VM_OBJ_LOCK(state->vmobject);
+
    pindex = os_pmap_getindex(pmap);
    if (pindex == (vm_pindex_t)-1) {
+      VM_OBJ_UNLOCK(state->vmobject);
       return NULL;
    }
 
@@ -505,6 +521,7 @@ os_kmem_alloc(int alloc_normal_failed) // IN
    if (!page) {
       os_pmap_putindex(pmap, pindex);
    }
+   VM_OBJ_UNLOCK(state->vmobject);
 
    return page;
 }
@@ -631,7 +648,12 @@ vmmemctl_poll(void *data) // IN
    if (!t->stop) {
       /* invoke registered handler, rearm timer */
       Balloon_QueryAndExecute();
+#if __FreeBSD_version >= 1300067
+      callout_reset(&t->callout_handle, BALLOON_POLL_PERIOD * hz, vmmemctl_poll,
+          t);
+#else
       t->callout_handle = timeout(vmmemctl_poll, t, BALLOON_POLL_PERIOD * hz);
+#endif
    }
 }
 
@@ -665,15 +687,23 @@ vmmemctl_init(void)
    }
 
    /* initialize timer state */
+#if __FreeBSD_version >= 1300067
+   callout_init(&state->timer.callout_handle, 0);
+#else
    callout_handle_init(&state->timer.callout_handle);
+#endif
 
    os_pmap_init(pmap);
    os_balloonobject_create();
 
    /* Set up and start polling */
-   callout_handle_init(&t->callout_handle);
    t->stop = FALSE;
+#if __FreeBSD_version >= 1300067
+   callout_reset(&t->callout_handle, BALLOON_POLL_PERIOD * hz, vmmemctl_poll,
+       t);
+#else
    t->callout_handle = timeout(vmmemctl_poll, t, BALLOON_POLL_PERIOD * hz);
+#endif
 
    vmmemctl_init_sysctl();
 
@@ -712,7 +742,11 @@ vmmemctl_cleanup(void)
 
    /* Stop polling */
    t->stop = TRUE;
+#if __FreeBSD_version >= 1300067
+   callout_drain(&t->callout_handle);
+#else
    untimeout(vmmemctl_poll, t, t->callout_handle);
+#endif
 
    os_balloonobject_delete();
    os_pmap_free(pmap);
@@ -783,8 +817,8 @@ vmmemctl_sysctl(SYSCTL_HANDLER_ARGS)
 
    /* format size info */
    len += snprintf(buf + len, sizeof(buf) - len,
-                   "target:             %8d pages\n"
-                   "current:            %8d pages\n",
+                   "target:             %8"FMT64"u pages\n"
+                   "current:            %8"FMT64"u pages\n",
                    stats->nPagesTarget,
                    stats->nPages);
 
@@ -847,7 +881,7 @@ vmmemctl_sysctl(SYSCTL_HANDLER_ARGS)
 static void
 vmmemctl_init_sysctl(void)
 {
-   oid =  sysctl_add_oid(NULL, SYSCTL_STATIC_CHILDREN(_vm), OID_AUTO,
+   oid =  SYSCTL_ADD_OID(NULL, SYSCTL_STATIC_CHILDREN(_vm), OID_AUTO,
                          BALLOON_NAME, CTLTYPE_STRING | CTLFLAG_RD,
                          0, 0, vmmemctl_sysctl, "A",
                          BALLOON_NAME_VERBOSE);
