@@ -1,5 +1,6 @@
 /*********************************************************
- * Copyright (C) 1998-2021 VMware, Inc. All rights reserved.
+ * Copyright (c) 1998-2024 Broadcom. All rights reserved.
+ * The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -93,7 +94,7 @@ VM_EMBED_VERSION(VMTOOLSD_VERSION_STRING);
 #if defined(_WIN32)
 #define GUESTINFO_DEFAULT_IFACE_EXCLUDES "vEthernet*"
 #else
-#define GUESTINFO_DEFAULT_IFACE_EXCLUDES "docker*,veth*,virbr*"
+#define GUESTINFO_DEFAULT_IFACE_EXCLUDES "docker*,veth*,virbr*,antrea-*,cali*"
 #endif
 
 /*
@@ -304,13 +305,13 @@ GuestInfoCheckIfRunningSlow(ToolsAppCtx *ctx)
        * Have a long enough delta to ensure that we have really missed a
        * collection.
        */
-      if (((int) delta * 1000) >= (2 * guestInfoPollInterval)) {
+      if (((int64)delta * 1000) >= ((int64) 2 * guestInfoPollInterval)) {
          gchar *msg, *rpcMsg;
 
          msg = g_strdup_printf(
                    "*** WARNING: GuestInfo collection interval longer than "
-                   "expected; actual=%d sec, expected=%d sec. ***\n",
-                   (int) delta, guestInfoPollInterval / 1000);
+                   "expected; actual=%"FMT64"d sec, expected=%d sec. ***\n",
+                   (int64) delta, guestInfoPollInterval / 1000);
 
          rpcMsg = g_strdup_printf("log %s", msg);
 
@@ -543,8 +544,13 @@ GuestInfoGather(gpointer data)
    int maxIPv4RoutesToGather;
    int maxIPv6RoutesToGather;
    gchar *osNameOverride;
-   gchar *osNameFullOverride;
+   gchar *osNameFullOverride = NULL;
    Bool maxNicsError = FALSE;
+   static uint32 logThrottleCount = 0;
+   Bool sendOsNames = FALSE;
+   char *osName = NULL;
+   char *osFullName = NULL;
+   char *detailedGosData = NULL;
 
    g_debug("Entered guest info gather.\n");
 
@@ -565,59 +571,59 @@ GuestInfoGather(gpointer data)
                                             CONFGROUPNAME_GUESTOSINFO,
                                             CONFNAME_GUESTOSINFO_SHORTNAME,
                                             NULL);
-
-   osNameFullOverride = VMTools_ConfigGetString(ctx->config,
-                                                CONFGROUPNAME_GUESTOSINFO,
-                                                CONFNAME_GUESTOSINFO_LONGNAME,
-                                                NULL);
-   /* If only the OS Full Name is provided, continue as normal, but emit
-    * warning. */
-   if (osNameOverride == NULL && osNameFullOverride != NULL) {
-      g_warning("Ignoring " CONFNAME_GUESTOSINFO_LONGNAME " override.\n");
-      g_warning("To use the GOS name override, "
-                CONFNAME_GUESTOSINFO_SHORTNAME " must be present in the "
-                "tools.conf file.\n");
-      g_free(osNameFullOverride);
+   /*
+    * CONFNAME_GUESTINFO_LONGNAME is ignored if CONFNAME_GUESTINFO_SHORTNAME
+    * is not provided
+    */
+   if (osNameOverride) {
+      osNameFullOverride = VMTools_ConfigGetString(ctx->config,
+                                                   CONFGROUPNAME_GUESTOSINFO,
+                                                   CONFNAME_GUESTOSINFO_LONGNAME,
+                                                   NULL);
    }
 
-   /* Only use override if at least the short OS name is provided */
-   if (osNameOverride == NULL) {
-      Bool sendOsNames = FALSE;
-      char *osName = NULL;
-      char *osFullName = NULL;
-      char *detailedGosData = NULL;
+   /* Gather all the relevant guest information. */
+   osFullName = Hostinfo_GetOSName();
+   osName = Hostinfo_GetOSGuestString();
 
-      /* Gather all the relevant guest information. */
-      osFullName = Hostinfo_GetOSName();
-      osName = Hostinfo_GetOSGuestString();
+   if (gSendDetailedGosData) {
+      detailedGosData = Hostinfo_GetOSDetailedData();
+   }
 
-      if (gSendDetailedGosData) {
-         detailedGosData = Hostinfo_GetOSDetailedData();
-      }
+   if (detailedGosData == NULL) {
+      g_debug("No detailed data.\n");
+      sendOsNames = TRUE;
+      gSendDetailedGosData = FALSE;
+   } else {
+      /* Build and attempt to send the detailed data */
+      HostinfoDetailedDataHeader *detailedDataHeader = NULL;
+      size_t infoHeaderSize;
+      size_t detailedGosDataLen;
+      size_t infoSize;
 
-      if (detailedGosData == NULL) {
-         g_debug("No detailed data.\n");
-         sendOsNames = TRUE;
-         gSendDetailedGosData = FALSE;
+      g_debug("Sending detailed data.\n");
+      detailedGosDataLen = strlen(detailedGosData);
+      infoHeaderSize = sizeof *detailedDataHeader;
+      infoSize = infoHeaderSize + detailedGosDataLen + 1; // cover NUL
+
+      detailedDataHeader = g_malloc(infoSize);
+      /* Clear struct and memory allocated for detailed data */
+      memset(detailedDataHeader, 0, infoSize);
+
+      /* Set the version of the detailed data header used */
+      detailedDataHeader->version = HOSTINFO_STRUCT_HEADER_VERSION;
+
+      if (osNameOverride) {
+         Str_Strcpy(detailedDataHeader->shortName, osNameOverride,
+                    sizeof detailedDataHeader->shortName);
+         if (osNameFullOverride == NULL) {
+            g_debug(CONFNAME_GUESTOSINFO_LONGNAME " was not set in "
+                    "tools.conf.\n");
+         } else {
+            Str_Strcpy(detailedDataHeader->fullName, osNameFullOverride,
+                       sizeof detailedDataHeader->fullName);
+         }
       } else {
-         /* Build and attempt to send the detailed data */
-         HostinfoDetailedDataHeader *detailedDataHeader = NULL;
-         size_t infoHeaderSize;
-         size_t detailedGosDataLen;
-         size_t infoSize;
-
-         g_debug("Sending detailed data.\n");
-         detailedGosDataLen = strlen(detailedGosData);
-         infoHeaderSize = sizeof *detailedDataHeader;
-         infoSize = infoHeaderSize + detailedGosDataLen + 1; // cover NUL
-
-         detailedDataHeader = g_malloc(infoSize);
-         /* Clear struct and memory allocated for detailed data */
-         memset(detailedDataHeader, 0, infoSize);
-
-         /* Set the version of the detailed data header used */
-         detailedDataHeader->version = HOSTINFO_STRUCT_HEADER_VERSION;
-
          if (osName == NULL) {
             g_warning("Failed to get OS name.\n");
          } else {
@@ -630,27 +636,46 @@ GuestInfoGather(gpointer data)
             Str_Strcpy(detailedDataHeader->fullName, osFullName,
                        sizeof detailedDataHeader->fullName);
          }
-
-         Str_Strcpy((char *)detailedDataHeader + infoHeaderSize,
-                    detailedGosData, infoSize - infoHeaderSize);
-
-         if (GuestInfoUpdateVMX(ctx, INFO_OS_DETAILED, detailedDataHeader,
-                                infoSize)) {
-            GuestInfoFreeDetailedData(gInfoCache.detailedData);
-            gInfoCache.detailedData = detailedDataHeader;
-            g_debug("Detailed data was sent successfully.\n");
-         } else {
-            /*
-             * Only send the OS Name if the VMX failed to receive the detailed
-             * data
-             */
-            gSendDetailedGosData = FALSE;
-            sendOsNames = TRUE;
-            g_debug("Detailed data was not sent successfully.\n");
-         }
       }
 
-      if (sendOsNames) {
+      Str_Strcpy((char *)detailedDataHeader + infoHeaderSize,
+                 detailedGosData, infoSize - infoHeaderSize);
+
+      if (GuestInfoUpdateVMX(ctx, INFO_OS_DETAILED, detailedDataHeader,
+                             infoSize)) {
+         GuestInfoFreeDetailedData(gInfoCache.detailedData);
+         gInfoCache.detailedData = detailedDataHeader;
+         g_debug("Detailed data was sent successfully.\n");
+      } else {
+         /*
+          * Only send the OS Name if the VMX failed to receive the detailed
+          * data
+          */
+         gSendDetailedGosData = FALSE;
+         sendOsNames = TRUE;
+         g_debug("Detailed data was not sent successfully.\n");
+      }
+   }
+
+   if (sendOsNames) {
+      if (osNameOverride) {
+         /* Use osName and osNameFull provided in config file */
+         if (osNameFullOverride == NULL) {
+            g_debug(CONFNAME_GUESTOSINFO_LONGNAME " was not set in "
+                    "tools.conf, using empty string.\n");
+         }
+         if (!GuestInfoUpdateVMX(ctx,
+                                 INFO_OS_NAME_FULL,
+                                 (osNameFullOverride == NULL) ? "" :
+                                 osNameFullOverride,
+                                 0)) {
+            g_warning("Failed to send INFO_OS_NAME_FULL\n");
+         }
+         if (!GuestInfoUpdateVMX(ctx, INFO_OS_NAME, osNameOverride, 0)) {
+            g_warning("Failed to send INFO_OS_NAME\n");
+         }
+         g_debug("Using values in tools.conf to override OS Name.\n");
+      } else {
          g_debug("Sending the short and long name\n");
          if (osFullName == NULL) {
             g_warning("Failed to get OS info.\n");
@@ -667,32 +692,14 @@ GuestInfoGather(gpointer data)
             }
          }
       }
-
-      free(detailedGosData);
-      free(osFullName);
-      free(osName);
-   } else {
-      /* Use osName and osNameFull provided in config file */
-      if (osNameFullOverride == NULL) {
-         g_warning(CONFNAME_GUESTOSINFO_LONGNAME " was not set in "
-                   "tools.conf, using empty string.\n");
-      }
-      if (!GuestInfoUpdateVMX(ctx,
-                              INFO_OS_NAME_FULL,
-                              (osNameFullOverride == NULL) ? "" :
-                                                             osNameFullOverride,
-                              0)) {
-         g_warning("Failed to send INFO_OS_NAME_FULL\n");
-      }
-      g_free(osNameFullOverride);
-
-      if (!GuestInfoUpdateVMX(ctx, INFO_OS_NAME, osNameOverride, 0)) {
-         g_warning("Failed to send INFO_OS_NAME\n");
-      }
-      g_free(osNameOverride);
-      g_debug("Using values in tools.conf to override OS Name.\n");
    }
 
+   free(detailedGosData);
+   free(osFullName);
+   free(osName);
+   g_free(osNameFullOverride);
+   g_free(osNameOverride);
+
 #if !defined(USERWORLD)
    disableQueryDiskInfo =
       g_key_file_get_boolean(ctx->config, CONFGROUPNAME_GUESTINFO,
@@ -767,8 +774,9 @@ GuestInfoGather(gpointer data)
       nicInfo = Util_SafeCalloc(1, sizeof (struct NicInfoV3));
    }
    if (maxNicsError) {
-      VMTools_VmxLog(ctx->rpc, "%s: NIC limit (%d) reached.",
-                     __FUNCTION__, NICINFO_MAX_NICS);
+      VMTools_VmxLogThrottled(&logThrottleCount, ctx->rpc,
+                              "%s: NIC limit (%d) reached.",
+                              __FUNCTION__, NICINFO_MAX_NICS);
    }
 
    /*
@@ -1604,6 +1612,12 @@ GuestInfoUpdateVMX(ToolsAppCtx *ctx,        // IN: Application context
          return FALSE;
       }
 
+      if (infoType == INFO_OS_NAME) {
+         g_message("Updated Guest OS name to %s\n", (char *) info);
+      } else if (infoType == INFO_OS_NAME_FULL) {
+         g_message("Updated Guest OS full name to %s\n", (char *) info);
+      }
+
       /* Update the value in the cache as well. */
       free(gInfoCache.value[infoType]);
       gInfoCache.value[infoType] = Util_SafeStrdup((char *) info);
@@ -1978,29 +1992,20 @@ TweakGatherLoop(ToolsAppCtx *ctx,
    gint pollInterval = 0;
 
    if (enable) {
-      pollInterval = defInterval * 1000;
-
       /*
        * Check the config registry for custom poll interval,
        * converting from seconds to milliseconds.
        */
-      if (g_key_file_has_key(ctx->config, CONFGROUPNAME_GUESTINFO,
-                             cfgKey, NULL)) {
-         GError *gError = NULL;
-
-         pollInterval = g_key_file_get_integer(ctx->config,
-                                               CONFGROUPNAME_GUESTINFO,
-                                               cfgKey, &gError);
-         pollInterval *= 1000;
-
-         if (pollInterval < 0 || gError) {
-            g_warning("Invalid %s.%s value. Using default %us.\n",
-                      CONFGROUPNAME_GUESTINFO, cfgKey, defInterval);
-            pollInterval = defInterval * 1000;
-         }
-
-         g_clear_error(&gError);
+      pollInterval = VMTools_ConfigGetInteger(ctx->config,
+                                              CONFGROUPNAME_GUESTINFO,
+                                              cfgKey, defInterval);
+      if (pollInterval < 0 || pollInterval > (G_MAXINT / 1000)) {
+         g_warning("Invalid %s.%s value. Using default %us.\n",
+                   CONFGROUPNAME_GUESTINFO, cfgKey, defInterval);
+         pollInterval = defInterval;
       }
+
+      pollInterval *= 1000;
    }
 
    if (*timeoutSource != NULL) {
@@ -2172,7 +2177,7 @@ GuestInfoServerConfReload(gpointer src,
  *
  ******************************************************************************
  */
-
+#if !defined(USERWORLD)
 static void
 GuestInfoServerIOFreeze(gpointer src,
                         ToolsAppCtx *ctx,
@@ -2181,7 +2186,7 @@ GuestInfoServerIOFreeze(gpointer src,
 {
    TweakGatherLoops(ctx, !freeze);
 }
-
+#endif
 
 /*
  ******************************************************************************
@@ -2372,7 +2377,9 @@ ToolsOnLoad(ToolsAppCtx *ctx)
       ToolsPluginSignalCb sigs[] = {
          { TOOLS_CORE_SIG_CAPABILITIES, GuestInfoServerSendCaps, NULL },
          { TOOLS_CORE_SIG_CONF_RELOAD, GuestInfoServerConfReload, NULL },
+#if !defined(USERWORLD)
          { TOOLS_CORE_SIG_IO_FREEZE, GuestInfoServerIOFreeze, NULL },
+#endif
          { TOOLS_CORE_SIG_RESET, GuestInfoServerReset, NULL },
          { TOOLS_CORE_SIG_SET_OPTION, GuestInfoServerSetOption, NULL },
          { TOOLS_CORE_SIG_SHUTDOWN, GuestInfoServerShutdown, NULL }
